#include "layout_edit.h"
#include "config.h"

//build the dialog
LayoutEdit::LayoutEdit( LayoutManager* l )
    : QWidget(0),
      lm(l),
      mainLayout(0),
      joypadTabs(0),
      cmbLayouts(0),
      btnAdd(0),
      btnRem(0),
      btnUpd(0),
      btnRev(0),
      btnExport(0),
      btnImport(0),
      btnRename(0) {
    setAttribute(Qt::WA_DeleteOnClose);
    setWindowTitle( THUNDERPAD_NAME );
    setWindowIcon(QPixmap(THUNDERPAD_ICON64));

    mainLayout = new QVBoxLayout(this);
    mainLayout->setSpacing(5);
    mainLayout->setMargin(5);

    QFrame* frame = new QFrame(this);
    frame->setFrameStyle(QFrame::Box | QFrame::Sunken);
    QGridLayout* g = new QGridLayout(frame);
    g->setMargin(5);
    g->setSpacing(5);
    cmbLayouts = new QComboBox(frame);
    connect(cmbLayouts, SIGNAL(activated(int)), this, SLOT(load(int)));

    //most of these buttons can link directly into slots in the LayoutManager
    btnAdd = new QPushButton(frame);
    btnAdd->setIcon(QIcon::fromTheme("list-add"));
    btnAdd->setToolTip("Add Layout");
    if (btnAdd->icon().isNull()) {
        btnAdd->setText("+");
    }
    btnAdd->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    connect(btnAdd, SIGNAL(clicked()), lm, SLOT(saveAs()));

    btnRem = new QPushButton(frame);
    btnRem->setIcon(QIcon::fromTheme("list-remove"));
    if (btnRem->icon().isNull()) {
        btnRem->setText("-");
    }
    btnRem->setToolTip("Remove Layout");
    btnRem->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    connect(btnRem, SIGNAL(clicked()), lm, SLOT(remove()));

    btnRename = new QPushButton("&Rename", frame);
    btnRename->setToolTip("Rename Layout");
    btnRename->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    connect(btnRename, SIGNAL(clicked()), lm, SLOT(rename()));

    QHBoxLayout *layoutLayout = new QHBoxLayout();
    layoutLayout->addWidget(cmbLayouts);
    layoutLayout->addWidget(btnAdd);
    layoutLayout->addWidget(btnRem);
    layoutLayout->addWidget(btnRename);
    mainLayout->addLayout(layoutLayout);

    btnImport = new QPushButton(QIcon::fromTheme("document-open"), "&Import", frame);
    connect(btnImport, SIGNAL(clicked()), lm, SLOT(importLayout()));
    g->addWidget(btnImport,1,0);

    btnExport = new QPushButton(QIcon::fromTheme("document-save-as"), "E&xport", frame);
    connect(btnExport, SIGNAL(clicked()), lm, SLOT(exportLayout()));
    g->addWidget(btnExport,1,1);

    btnUpd = new QPushButton(QIcon::fromTheme("document-save"), "&Save", frame);
    connect(btnUpd, SIGNAL(clicked()), lm, SLOT(save()));
    g->addWidget(btnUpd,1,2);

    btnRev = new QPushButton(QIcon::fromTheme("document-revert"), "Re&vert", frame);
    connect(btnRev, SIGNAL(clicked()), lm, SLOT(reload()));
    g->addWidget(btnRev,1,3);
    mainLayout->addWidget( frame );

    //create a tab widget for joystick devices - much more accessible
    joypadTabs = new QTabWidget(this);
    joypadTabs->setTabPosition(QTabWidget::North);
    mainLayout->addWidget(joypadTabs);

    //go through each of the available joysticks
    foreach (JoyPad *joypad, lm->available) {
        connect(this, SIGNAL(focusStateChanged(bool)), joypad, SLOT(focusChange(bool)));
        //create accessible widget and add as tab
        QScrollArea *scroll = new QScrollArea();
        scroll->setWidgetResizable(true);
        JoyPadWidget *widget = joypad->widget(scroll, joypad->getIndex());
        scroll->setWidget(widget);
        joypadTabs->addTab(scroll, joypad->getName());
    }

    updateLayoutList();

    //add the buttons at the bottom.
    QPushButton* close = new QPushButton(QIcon::fromTheme("window-close"), "&Close Dialog", this );
    connect(close, SIGNAL(clicked()), this, SLOT(close()));

    QPushButton* quit = new QPushButton(QIcon::fromTheme("application-exit"), "&Quit", this );
    connect( quit, SIGNAL( clicked() ), qApp, SLOT(quit()));

    QHBoxLayout* h = new QHBoxLayout();
    h->setMargin(0);
    h->setSpacing(5);
    h->addWidget(close);
    h->addWidget(quit);
    mainLayout->addLayout(h);

    connect(qApp, SIGNAL(focusChanged ( QWidget * , QWidget *  ) ), this, 
        SLOT(appFocusChanged(QWidget *, QWidget *)));

    this->show();
}

void LayoutEdit::setLayout(const QString &layout) {
    //change the selection
    for (int i = 0; i < cmbLayouts->count(); ++ i) {
        if (cmbLayouts->itemData(i).toString() == layout) {
            cmbLayouts->setCurrentIndex(i);
            break;
        }
    }

    bool hasLayout = !layout.isNull();
    btnRem->setEnabled(hasLayout);
    btnRename->setEnabled(hasLayout);

    //update all the JoyPadWidgets.
    for (int i = 0; i < joypadTabs->count(); i++) {
        QScrollArea *scroll = qobject_cast<QScrollArea*>(joypadTabs->widget(i));
        if (scroll) {
            JoyPadWidget *jpw = qobject_cast<JoyPadWidget*>(scroll->widget());
            if (jpw) {
                jpw->update();
            }
        }
    }
}

void LayoutEdit::updateLayoutList() {
    //blank the list, then load in new names from the LayoutManager.
    cmbLayouts->clear();
    cmbLayouts->addItem("[NO LAYOUT]", QVariant(QString()));
    if (lm->currentLayout.isNull()) {
        cmbLayouts->setCurrentIndex(0);
    }
    foreach (const QString& layout, lm->getLayoutNames()) {
        cmbLayouts->addItem(layout, layout);
        if (layout == lm->currentLayout) {
            cmbLayouts->setCurrentIndex(cmbLayouts->count() - 1);
        }
    }
    //Update layout list for all button edit widgets
    for (int i = 0; i < joypadTabs->count(); i++) {
        const QStringList layoutNames = lm->getLayoutNames();
        QScrollArea *scroll = qobject_cast<QScrollArea*>(joypadTabs->widget(i));
        if (scroll) {
            JoyPadWidget *jpw = qobject_cast<JoyPadWidget*>(scroll->widget());
            if (jpw) {
                jpw->updateButtonLayoutLists(layoutNames);
            }
        }
    }
}

void LayoutEdit::updateJoypadWidgets() {
    // Clear existing tabs
    while (joypadTabs->count() > 0) {
        QWidget *widget = joypadTabs->widget(0);
        joypadTabs->removeTab(0);
        widget->deleteLater();
    }
    
    // Add tabs for each available joypad
    foreach (JoyPad *joypad, lm->available) {
        connect(this, SIGNAL(focusStateChanged(bool)), joypad, SLOT(focusChange(bool)));
        // Create accessible widget and add as tab
        QScrollArea *scroll = new QScrollArea();
        scroll->setWidgetResizable(true);
        JoyPadWidget *widget = joypad->widget(scroll, joypad->getIndex());
        scroll->setWidget(widget);
        joypadTabs->addTab(scroll, joypad->getName());
    }
}

void LayoutEdit::appFocusChanged(QWidget *old, QWidget *now) {
    if (now != NULL && old == NULL) {
        emit focusStateChanged(false);
    } else if(old != NULL && now == NULL) {
        emit focusStateChanged(true);
        foreach (JoyPad *joypad, lm->available) {
            debug_mesg("iterating and releasing\n");
            joypad->release();
        }
        debug_mesg("done releasing!\n");
    }
}

void LayoutEdit::load(int index) {
    lm->load(cmbLayouts->itemData(index).toString());
}
